@@ -13,11 +13,11 @@ interface IInputSignals {
   dsr: boolean;
 }
 interface ISerialOptions {
-  baudRate?: number; /* >0 */
-  dataBits?: number; /*ONLY: 7 8 */
+  baudRate?: number; // >0
+  dataBits?: number; //ONLY: 7 8
   parity?: 'none' | 'even' | 'odd';
-  stopBits?: number; /*ONLY: 1 2 */
-  bufferSize?: number; /* >0 */
+  stopBits?: number; //ONLY: 1 2
+  bufferSize?: number; // >0
 }
 interface ISerialPortInfo {
   serialNumber?: string;
@@ -30,7 +30,7 @@ interface ISerialPortInfo {
 }
 interface IRequestOption {
   usbVendorId?: number;
-  usbProductId?: number; /* usbVendorId required if productID present */
+  usbProductId?: number; // usbVendorId required if productID present
 }
 interface IRequestOptions {
   filters?: IRequestOption[];
@@ -45,8 +45,8 @@ interface ISerialPort {
   getInfo: () => ISerialPortInfo;
 }
 interface ISerial extends EventTarget {
-  onconnect: ((this: ISerial, ev: Event) => any) | null; /* SerialConnectionEvent */
-  ondisconnect: ((this: ISerial, ev: Event) => any) | null; /* SerialConnectionEvent */
+  onconnect: ((this: ISerial, ev: Event) => any) | null; // SerialConnectionEvent
+  ondisconnect: ((this: ISerial, ev: Event) => any) | null; // SerialConnectionEvent
   getPorts: () => Promise<ISerialPort[]>;
   requestPort: (options?: IRequestOptions) => Promise<ISerialPort>;
 }
@@ -56,8 +56,9 @@ interface INavigatorSerial extends Navigator {
 
 export class SerialHubPort {
   port: ISerialPort | null;
-  outputStream: WritableStream | null;
-  outputDone: Promise<void> | null;
+  //outputStream: WritableStream | null;
+  //outputDone: Promise<void> | null;
+  writer: WritableStreamDefaultWriter | null;
   //inputStream: ReadableStream |null;
   //inputDone: Promise<void> |null;
   reader: ReadableStreamDefaultReader | null;
@@ -67,15 +68,16 @@ export class SerialHubPort {
       oldSP.disconnect(); //Dispose of prior "port" if passed to us
     }
     this.port = null;
-    this.outputStream = null;
-    this.outputDone = null;
+    //this.outputStream = null;
+    //this.outputDone = null;
+    this.writer = null;
     //this.inputStream = null;
     //this.inputDone = null;
     this.reader = null;
   }
-  
-  async connect() {
-    const NAV: INavigatorSerial = (window.navigator as INavigatorSerial);
+
+  async connect(): Promise<void> {
+    const NAV: INavigatorSerial = window.navigator as INavigatorSerial;
     if (!NAV || !NAV.serial) {
       return;
     }
@@ -90,11 +92,12 @@ export class SerialHubPort {
     this.port = rawPort;
     await this.port.open({ baudRate: 115200 });
 
-    const encoder = new TextEncoderStream();
-    this.outputDone = encoder.readable.pipeTo(this.port.writable);
-    this.outputStream = encoder.writable;
+    //const encoder = new TextEncoderStream();
+    //this.outputDone = encoder.readable.pipeTo(this.port.writable);
+    //this.outputStream = encoder.writable;
+    this.writer = this.port.writable.getWriter();
 
-    //let decoder = new TextDecoderStream();
+    //const decoder = new TextDecoderStream();
     //this.inputDone = this.port.readable.pipeTo(decoder.writable);
     //this.inputStream = decoder.readable;
     //this.reader = this.inputStream.getReader();
@@ -103,44 +106,42 @@ export class SerialHubPort {
     console.log('CONNECT: ', this);
     //Let cbConnect initiate this.readLoop(f);
   }
-  
-  async disconnect() {
-    console.log("CLOSE: ", this);
+
+  async disconnect(): Promise<void> {
+    console.log('CLOSE: ', this);
     if (this.reader) {
       await this.reader.cancel();
       this.reader = null;
       //if (this.inputDone) await this.inputDone.catch(() => {});
       //this.inputDone = null;
     }
-    if (this.outputStream) {
-      await this.outputStream.getWriter().close();
-      await this.outputDone;
-      this.outputStream = null;
-      this.outputDone = null;
+    if (this.writer) {
+      this.writer.close();
+      this.writer = null;
+      //await this.outputStream.getWriter().close();
+      //await this.outputDone;
+      //this.outputStream = null;
+      //this.outputDone = null;
     }
     if (this.port) {
       await this.port.close();
       this.port = null;
     }
   }
-  
-  writeToStream(...lines: string[]) {
-    if (!this.outputStream) {
-      return;
+
+  writeToStream(data: ArrayBufferView[] | ArrayBuffer[]): void {
+    if (this.writer) {
+      data.forEach(async (d: any) => {
+        //Anonymous function is ASYNC so it can AWAIT the write() call below
+        console.log('[WRITE]', d);
+        await this.writer?.write(d); // AWAIT in sequence, to avoid parallel promises
+      });
     }
-    const writer = this.outputStream.getWriter();
-    lines.forEach(line => {
-      console.log('[SEND]', line);
-      writer.write(line);
-    });
-    writer.releaseLock();
   }
-  
-  async readLoop(cbRead: Function) {
-    while (true) {
-      if (!this.reader) {
-        break;
-      }
+
+  async readLoop(cbRead: (theVAL: any) => void): Promise<void> {
+    // eslint-disable-next-line no-constant-condition
+    while (this.reader) {
       const { value, done } = await this.reader.read();
       if (value) {
         console.log('[readLoop] VALUE', value);
@@ -166,6 +167,7 @@ export class SerialHubPort {
     return true;
   }
 
+  /*
   static test(f: Function): SerialHubPort {
     const W: any = window as any;
     const SER = new SerialHubPort(W.serPort);
@@ -177,9 +179,10 @@ export class SerialHubPort {
     });
     return SER;
   }
+  */
 
-  static createHub(cbConnect: Function): SerialHubPort{
-    const W: any = (window as any);
+  static createHub(cbConnect: (theSER: SerialHubPort) => void): SerialHubPort {
+    const W: any = window as any;
     const oldSER = W.serPort;
     const SER = new SerialHubPort(oldSER);
     W.serPort = SER; //Assign to a global location
