@@ -11,8 +11,7 @@ import {
 import { MODULE_NAME, MODULE_VERSION } from './version';
 
 // Import the CSS
-//import '../css/widget.css'
-import '../style/widget.css';
+import '../style/index.css'; //was '../css/widget.css'
 
 import * as utils from '@jupyter-widgets/base';
 import { UUID } from '@lumino/coreutils';
@@ -20,8 +19,9 @@ import { UUID } from '@lumino/coreutils';
 import { SerialHubPort } from './webseriallink';
 
 export class SerialHubModel extends DOMWidgetModel {
-  defaults() {
-    return {...super.defaults(),
+  defaults(): any {
+    return {
+      ...super.defaults(),
       _model_name: SerialHubModel.model_name,
       _model_module: SerialHubModel.model_module,
       _model_module_version: SerialHubModel.model_module_version,
@@ -32,9 +32,13 @@ export class SerialHubModel extends DOMWidgetModel {
       isSupported: false,
       status: 'Initializing...',
       value: 'Loading...',
+      pkt_recv_front: 0,
+      pkt_recv_back: 0,
+      pkt_send_front: 0,
+      pkt_send_back: 0
     };
   }
-  
+
   private static _mytempid: string = utils.uuid();
   static get mytempid(): string {
     return SerialHubModel._mytempid;
@@ -48,23 +52,27 @@ export class SerialHubModel extends DOMWidgetModel {
   static model_name = 'SerialHubModel';
   static model_module = MODULE_NAME;
   static model_module_version = MODULE_VERSION;
-  static view_name = 'SerialHubView';   // Set to null if no view
-  static view_module = MODULE_NAME;   // Set to null if no view
+  static view_name = 'SerialHubView'; // Set to null if no view
+  static view_module = MODULE_NAME; // Set to null if no view
   static view_module_version = MODULE_VERSION;
 }
 
 export class SerialHubView extends DOMWidgetView {
-  _el_status: HTMLDivElement | null = null;
+  _el_status: HTMLSpanElement | null = null;
+  _el_stats: HTMLSpanElement | null = null;
   _el_value: HTMLPreElement | null = null;
+  _shp: SerialHubPort | null = null;
 
   render(): this {
     this.el.id = this.id || UUID.uuid4();
     this.el.classList.add('xx-serialhub-widget');
 
     /* Create a couple sub-Elements for our custom widget */
-    this._el_status = window.document.createElement("div");
+    this._el_status = window.document.createElement('span');
     this._el_status.classList.add('xx-serialhub-status');
-    this._el_value = window.document.createElement("pre");
+    this._el_stats = window.document.createElement('span');
+    this._el_stats.classList.add('xx-serialhub-stats');
+    this._el_value = window.document.createElement('pre');
     this._el_value.classList.add('xx-serialhub-value');
 
     /* Click events wrapped to capture "this" object */
@@ -72,12 +80,17 @@ export class SerialHubView extends DOMWidgetView {
     this._el_value.onclick = (ev: MouseEvent) => this.click_value(ev);
 
     /* Maybe is more appropriate append() function availablie? */
-    this.$el.append(this._el_status, this._el_value);
+    this.el.append(this._el_status, this._el_stats, this._el_value);
 
     this.changed_status();
     this.changed_value();
+    this.changed_stats();
     this.model.on('change:status', this.changed_status, this);
     this.model.on('change:value', this.changed_value, this);
+    this.model.on('change:pkt_recv_front', this.changed_stats, this);
+    this.model.on('change:pkt_recv_back', this.changed_stats, this);
+    this.model.on('change:pkt_send_front', this.changed_stats, this);
+    this.model.on('change:pkt_send_back', this.changed_stats, this);
 
     this.model.on('msg:custom', this.msg_custom, this);
 
@@ -87,58 +100,69 @@ export class SerialHubView extends DOMWidgetView {
       SerialHubPort.isSupported() ? 'Supported' : 'Unsupported'
     );
     this.touch();
-
     return this;
   }
 
-  changed_status() : void {
-    if (!this._el_status) {
-      return;
+  changed_status(): void {
+    if (this._el_status && this.model) {
+      this._el_status.textContent = this.model.get('status');
     }
-    this._el_status.textContent = this.model.get('status');
   }
-  changed_value() : void {
-    if (!this._el_value) {
-      return;
+  changed_value(): void {
+    if (this._el_value && this.model) {
+      this._el_value.textContent = this.model.get('value');
     }
-    this._el_value.textContent = this.model.get('value');
   }
-
-  click_status_OLD(this: SerialHubView, ev: MouseEvent): void {
-    //console.log(this, arguments, this.model);
-    let SHP = SerialHubPort.test((value: any) => {
-      console.log(value);
-      this.model.send({ type: 'binary' }, {}, [value]);
-    });
-    console.log('DONE_OLD', SHP);
+  changed_stats(): void {
+    if (this._el_stats) {
+      let stats = '';
+      stats += 'Rf:' + this.model.get('pkt_recv_front');
+      stats += ' Rb:' + this.model.get('pkt_recv_back');
+      stats += ' Sf:' + this.model.get('pkt_send_front');
+      stats += ' Sb:' + this.model.get('pkt_send_back');
+      this._el_stats.textContent = stats;
+    }
   }
 
   click_status(this: SerialHubView, ev: MouseEvent): void {
-    console.log('click_status', this, arguments, this.model);
-    let SHP = SerialHubPort.createHub((theSHP: SerialHubPort) => {
+    console.log('click_status', this, this.model, ev);
+    this._shp = SerialHubPort.createHub((theSHP: SerialHubPort) => {
       console.log('theSHP', theSHP);
       theSHP.readLoop((value: any) => {
         console.log('DATA-IN', value);
-        this.model.send({ type: 'binary' }, {}, [value]);
+        this.model.send({ type: 'RECV' }, {}, [value]);
+        const cnt: number = this.model.get('pkt_recv_front') + 1;
+        this.model.set('pkt_recv_front', cnt);
       });
     });
-    console.log('DONE5', SHP);
+    console.log('DONE click', this._shp);
   }
 
   click_value(this: SerialHubView, ev: MouseEvent): void {
     if (!this || !this.model) {
       return;
     }
-    this.model.send({ type: 'text', text: 'VALUE6\n' }, {}, []);
-    (window as any).serPort.writeToStream('6');
+    this.model.send({ type: 'text', text: 'VALUE-6\n' }, {}, []);
+    const encoder = new TextEncoder();
+    const theData = encoder.encode('6');
+    this._shp?.writeToStream([theData]);
+    this.model.set('pkt_send_front', this.model.get('pkt_send_front') + 1);
   }
 
   msg_custom(this: SerialHubView, mData: Dict<any>, mBuffs: DataView[]): void {
-    console.log(this, mData, mBuffs);
+    //console.log(this, mData, mBuffs);
     const msgType = mData['type'];
-    if (msgType == 'text') {
-      (window as any).serPort.writeToStream(mData['text']);
+    if (msgType === 'SEND') {
+      console.log('MSG-SEND', mBuffs);
+      this._shp?.writeToStream(mBuffs);
+      this.model.set('pkt_send_front', this.model.get('pkt_send_front') + 1);
+    } else if (msgType === 'SEND2') {
+      const encoder = new TextEncoder();
+      const theData = encoder.encode(mData['text']);
+      this._shp?.writeToStream([theData]);
+      this.model.set('pkt_send_front', this.model.get('pkt_send_front') + 1);
+    } else {
+      console.log('UNKNOWN MESSAGE: ', msgType, mData, mBuffs);
     }
   }
-
 }
